import chainlit as cl
from bigdl.llm.transformers import AutoModelForCausalLM
from transformers import AutoTokenizer,TextIteratorStreamer
from threading import Thread
import torch

print("Begin to load the model ......")
model_path = "./llama2-7b-bigdl-llm-INT4"
model = AutoModelForCausalLM.load_low_bit(model_path)

print("Begin to load the tokenizer ......")
tokenizer = AutoTokenizer.from_pretrained("./llama2-7b")
tokenizer.pad_token = tokenizer.eos_token

@cl.on_message
async def main(message: cl.Message):

    prompt = "<s>Human: " + message.content + "\n</s><s>Assistant: "
    input_ids = tokenizer(prompt, return_tensors="pt", add_special_tokens=False).input_ids[:,-512:]
    streamer = TextIteratorStreamer(tokenizer,skip_prompt=True)
    generate_input = {
        "input_ids":input_ids,
        "max_new_tokens":512,
        "do_sample":True,
        "top_k":50,
        "top_p":0.95,
        "temperature":0.3,
        "repetition_penalty":1.3,
        "streamer":streamer,
        "eos_token_id":tokenizer.eos_token_id,
        "bos_token_id":tokenizer.bos_token_id,
        "pad_token_id":tokenizer.pad_token_id
    }

    thread = Thread(target=model.generate, kwargs=generate_input)
    thread.start()
    msg = cl.Message(content="")
    await msg.send()

    for new_text in streamer:
        if len(new_text)==0 or new_text=='</s>':
            continue
        await msg.stream_token(new_text)
